package jp.co.celestee.ai.interfaces.body;

import java.util.ArrayList;
import java.util.List;

import jp.co.celestee.ai.core.instance.PhysicalDesireKeyManager;
import jp.co.celestee.ai.core.signal.PleasureIF;
import jp.co.celestee.ai.interfaces.AbstractSubstance;
import jp.co.celestee.ai.interfaces.body.pleasure.Palate;
import jp.co.celestee.ai.interfaces.body.pleasure.ReceivePleasure;
import jp.co.celestee.ai.interfaces.constant.PhysicalDesireConst;
import jp.co.celestee.ai.interfaces.input.AbstractPhysicalSensibleSignal;
import jp.co.celestee.ai.interfaces.input.DefaultDataCooperationSignal;
import jp.co.celestee.ai.interfaces.input.DefaultEnvironmentSensibleSignal;
import jp.co.celestee.ai.interfaces.input.DefaultPhysicalSensibleSignal;
import jp.co.celestee.external.rule.RuleDataIF;
import jp.co.celestee.external.rule.RuleIF;
import jp.co.celestee.external.rule.RuleManager;
import jp.co.celestee.external.rule.dinamic.DynamicRule;
import jp.co.celestee.external.rule.dinamic.DynamicRuleAccessorIF;
import jp.co.celestee.external.rule.dinamic.DynamicRuleData;

public class BodySubstance extends AbstractSubstance implements DynamicRuleAccessorIF {

	private static BodySubstance subInstance = new BodySubstance();
	
	private int _energy = 1;

	private int _maxEnergy = 10;
	
	private int _walkCount = 0;

	private BodySubstance() {}
	
	public static BodySubstance getInstance() {
		return subInstance;
	}

	/**
	 * [IFւ̓
	 */
	@Override
	public void inputDynamic(DynamicRuleData ruleData) {
		inputDataFromRule(ruleData);
	}

	/**
	 * [IFւ̓
	 */
	@Override
	public void inputFromRule(RuleDataIF ruleData, Signals signals) {

		DynamicRuleData dynamicRuleData = (DynamicRuleData)ruleData;

		//IFŃf[^ϊ
		signals.setDataCooperationSignal(BodyDataConverter.change(dynamicRuleData));
		
	}

	/**
	 * AI̓
	 */
	@Override
	public void inputFromAI(Object aiData, Signals signals) {
		signals.setDataCooperationSignal((DefaultDataCooperationSignal)aiData);
	}

	/**
	 * C
	 * I~̒͂ōs
	 */
	@Override
	public void process(Signals signals) {
		
		//y̔(gOUTPUTINTOANV͔Ȃ)
		BodySignalData data = ((BodySignalData)signals.getDataCooperationSignal().getSignalData());
		if(data.getFeel() == BodyFeelConst.INTO) {
			
			if(data.getPain() == PleasureIF.PLEASURE_LEVEL) {
				DefaultEnvironmentSensibleSignal esSignal = new DefaultEnvironmentSensibleSignal();
				esSignal.setElectricPotential(signals.getDataCooperationSignal().getElectricPotential());
				esSignal.setPleasureLevel(PleasureIF.PLEASURE_LEVEL);
				signals.setEnvironmentSensibleSignal(esSignal);
				
				System.out.println("!!! (I̊) !!!");
				
				//yf[^̐ݒ
				PleasureIF pleasure = new Palate(1, 0, 0, 0, 0);
				pleasure.setCauseOutputId(PhysicalDesireKeyManager.getPhysicalOutputId(PhysicalDesireConst.EAT_DESIRE));
				esSignal.setPleasure(pleasure);

				_energy += 10;
				_energy = _maxEnergy < _energy ? _maxEnergy : _energy;
			}
			else if(data.getPain() == PleasureIF.DISPLEASURE_LEVEL) {
				DefaultEnvironmentSensibleSignal esSignal = new DefaultEnvironmentSensibleSignal();
				esSignal.setElectricPotential(signals.getDataCooperationSignal().getElectricPotential());
				esSignal.setPleasureLevel(PleasureIF.DISPLEASURE_LEVEL);
				signals.setEnvironmentSensibleSignal(esSignal);
				
				System.out.println("!!! ܂(I̊) !!!");
				
				//sf[^̐ݒ
				PleasureIF pleasure = new Palate(0, 0, 0, 1, 0);
				pleasure.setCauseOutputId(PhysicalDesireKeyManager.getPhysicalOutputId(PhysicalDesireConst.EAT_DESIRE));
				esSignal.setPleasure(pleasure);

				_energy += 10;
				_energy = _maxEnergy < _energy ? _maxEnergy : _energy;
			}
			
		}
		else if(data.getFeel() == BodyFeelConst.WALK) {
			_walkCount += 1;
			_energy = _walkCount % 3 == 0 ? _energy - 1 : _energy;
			_energy = _energy < 0 ? 0 : _energy;
		}
		else if(data.getFeel() == BodyFeelConst.RECEIVE) {
			
			DefaultEnvironmentSensibleSignal esSignal = new DefaultEnvironmentSensibleSignal();
			esSignal.setElectricPotential(signals.getDataCooperationSignal().getElectricPotential());
			esSignal.setPleasureLevel(PleasureIF.PLEASURE_LEVEL);
			signals.setEnvironmentSensibleSignal(esSignal);
			
			System.out.println("!!! E  !!!  " + signals.getDataCooperationSignal().getElectricPotential());
			
			//yf[^̐ݒ
			PleasureIF pleasure = new ReceivePleasure(0);
			pleasure.setCauseOutputId(PhysicalDesireKeyManager.getPhysicalOutputId(PhysicalDesireConst.PICKUP_DESIRE));
			esSignal.setPleasure(pleasure);

		}
		
		//I~̒ʒm(ɏ󋵂m点)
		List<AbstractPhysicalSensibleSignal> psSignalList = getPhysicalSensibleSignalList();

		for(AbstractPhysicalSensibleSignal psSignal : psSignalList) {
			signals.addPhysicalSensibleSignalList(psSignal);
		}

	}

	/**
	 * [ւ̏o
	 */
	@Override
	public void outputToRule(Signals signals) {

		RuleIF rule = RuleManager.getInstance().getRule(DynamicRule.DYNAMIC_RULE);
		//[֓
		DynamicRuleData ruleData = BodyDataConverter.change(signals.getDataCooperationSignal());
		ruleData.setObject(this);
		rule.input(ruleData);

	}
	
	/**
	 * I~̏󋵂擾܂
	 * @return
	 */
	public List<AbstractPhysicalSensibleSignal> getPhysicalSensibleSignalList() {
		
		List<AbstractPhysicalSensibleSignal> list = new ArrayList<AbstractPhysicalSensibleSignal>();
		
		//IH~
		int iWantToEat = _maxEnergy - _energy;
		DefaultPhysicalSensibleSignal psSignal1 = new DefaultPhysicalSensibleSignal();
		psSignal1.setElectricPotential(iWantToEat);
		psSignal1.setKey(PhysicalDesireConst.EAT_DESIRE);
		list.add(psSignal1);
		
		//IE~
		DefaultPhysicalSensibleSignal psSignal2 = new DefaultPhysicalSensibleSignal();
		psSignal2.setElectricPotential(8);
		psSignal2.setKey(PhysicalDesireConst.PICKUP_DESIRE);
		list.add(psSignal2);
		
		return list;
	}
	
	public int getEnergy() {
		return _energy;
	}

	public int getMaxEnergy() {
		return _maxEnergy;
	}
}
